#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    1.0          05 Sep 2006

# saltbias...

from pyraf import iraf 
from pyraf.iraf import pysalt
from pylab import *
import saltprint, saltstat, saltkey, saltfit, saltio
import os, string, sys, glob, pyfits, time

# -----------------------------------------------------------
# core routine

def saltbias(images,outimages,outpref,subover,trim,subbias,crbias,masterbias,
	     median,function,order,rej_lo,rej_hi,niter,plotover,turbo,logfile,
	     clobber,verbose,status):

# initialize lists and variables

    status = 0
    ifil = 0
    ii = 0
    mbiasdata = []
    bstruct = ''
    biasgn = ''
    biassp = ''
    biasbn = ''
    biasin = ''
    filetime = {}
    biastime = {}
    for i in range(1,7):
	filetime[i] = []
	biastime[i] = []

# test the logfile

    logfile = saltio.logname(logfile)

# log the call 

    saltprint.line(logfile,verbose)
    history = 'SALTBIAS -- '
    history += 'images='+images+' '
    history += 'outimages='+outimages+' '
    history1 = 'outpref='+outpref+' '
    yn = 'n'
    if (subover): yn = 'y'
    history1 += 'subover='+yn+' '
    yn = 'n'
    if (trim): yn = 'y'
    history1 += 'trim='+yn+' '
    yn = 'n'
    if (subbias): yn = 'y'
    history1 += 'subbias='+yn+' '
    yn = 'n'
    if (crbias): yn = 'y'
    history1 += 'crbias='+yn+' '
    yn = 'n'
    history1 += 'masterbias='+masterbias+' '
    if (median): yn = 'y'
    history1 += 'median='+yn+' '
    history1 += 'function='+function+' '
    history1 += 'order='+str(order)+' '
    history1 += 'rej_lo='+str(rej_lo)+' '
    history1 += 'rej_hi='+str(rej_hi)+' '
    history1 += 'niter='+str(niter)+' '
    yn = 'n'
    if (plotover): yn = 'y'
    history1 += 'plotover='+yn+' '
    yn = 'n'
    if (turbo): yn = 'y'
    history1 += 'turbo='+yn+' '
    history1 += 'logfile='+logfile+' '
    yn = 'n'
    if (clobber): yn = 'y'
    history1 += 'clobber='+yn+' '
    yn = 'n'
    if (verbose): yn = 'y'
    history1 += 'verbose='+yn
    history +=  history1
    saltprint.log(logfile,history,verbose)
    if (niter == 0): niter += 1

# start time

    saltprint.time('\nSALTBIAS -- started at',logfile,verbose)

# is the input file specified?
   
    if (status == 0): status = saltio.filedefined('Input',images,logfile)

# if the input file is a list, does it exist?

    if (status == 0 and images[0] == '@'): 
        status = saltio.listexists('Input',images,logfile)

# parse list of input files

    if (status == 0):
        infiles, status = saltio.listparse('Raw image',images,'','','',logfile)

# check input files exist

    if (status == 0): status = saltio.filesexist(infiles,'','r',logfile)

# load output name list: @list, * and comma separated
   
    outimages = outimages.strip()
    outpref = outpref.strip()
    if (status == 0 and len(outpref) == 0 and len(outimages) == 0):
        message = 'ERROR -- SALTGAIN: output file(s) not specified'
        status = saltprint.err(logfile,message)

# test output @filelist exists

    if (status == 0 and len(outimages) > 0 and outimages[0] == '@'): 
        status = saltio.listexists('Output',outimages,logfile)

# parse list of output files

    if (status == 0):
        outfiles, status = saltio.listparse('Output image',outimages,outpref,infiles,'',logfile)

# are input and output lists the same length?

    if (status == 0): status = saltio.comparelists(infiles,outfiles,'Input','output',logfile)

# do the output files already exist?

    if (status == 0 and not clobber): status = saltio.filesexist(outfiles,'','w',logfile)

# Incompatible bias options?

    if (status == 0 and subbias and crbias):
	message = 'ERROR -- SALTBIAS: the master bias frame must be created '
	message += 'before it can be subtracted\n'
	status = saltprint.err(logfile,message)
    if (status == 0 and subover and subbias and function != 'polynomial'):
	message = 'ERROR -- SALTBIAS/COLBIAS: the master bias cannot be subtracted '
	message += 'when function != polynomial and subover=yes\n'
	status = saltprint.err(logfile,message)
    if (status == 0 and not trim and function != 'polynomial'):
	message = 'ERROR -- SALTBIAS/COLBIAS: images must be trimmed if function != polynomial\n'
	status = saltprint.err(logfile,message)

# Does master bias frame exist?

    if (status == 0):
	message = 'ERROR -- SALTBIAS: Master bias frame ' + masterbias
	if (os.path.isfile(masterbias)):
	    if (crbias and not clobber):
		message += ' already exists'
		status = saltprint.err(logfile,message)
	else:
	    if (subbias):
		message += ' does not exist'
		status = saltprint.err(logfile,message)		

# open master bias frame

    if (status == 0 and subbias):
	bstruct, status = saltio.openfits(masterbias,logfile)

# gain, speed, binning and instrument of master bias frame

    if (subbias):
	if (status == 0): biasgn, status = saltkey.get('GAINSET',bstruct[0],masterbias,logfile)
	if (status == 0): biassp, status = saltkey.get('ROSPEED',bstruct[0],masterbias,logfile)
	if (status == 0): biasbn, status = saltkey.get('CCDSUM',bstruct[0],masterbias,logfile)
	if (status == 0): biasin, status = saltkey.get('INSTRUME',bstruct[0],masterbias,logfile)

# open each raw image file

    for imageitem in infiles:
	if (status == 0):
	    infile = imageitem
	    outfile = outfiles[ifil]
	    struct, status = saltio.openfits(infile,logfile)

# clobber old files
	    
	if (status == 0 and clobber and os.path.isfile(outfile)): 
	    status = saltio.delete(outfile,False,logfile)

# subtract bias

	if (status == 0): 
	    history = 'SALTBIAS -- '
            history += 'images='+infile+' '
            history += 'outimages='+outfile+' '
            history +=  history1
	    struct,mbiasdata,biasbn,biassp,biasbn,biasgn,filetime,biastime, \
		amplifiers,instrume,status = \
		bias(struct,infile,outfile,ifil,subbias,crbias,subover,plotover,median,
		     function,order,rej_lo,rej_hi,niter,trim,masterbias,bstruct,
		     mbiasdata,biasgn,biassp,biasbn,biasin,filetime,biastime,
		     turbo,history1,logfile,verbose,status)

# write FITS file		

	if (status == 0): filename = outfile.split('/')[-1]
	if (status == 0 and function == 'polynomial'): status = saltio.writefits(struct,outfile,logfile)
	if (status == 0): status = saltio.closefits(struct,logfile)
	if (status == 0): saltprint.time('\nSALTBIAS -- created ' + filename + ' at',logfile,verbose)
	ifil += 1

# clobber old master bias file

    if (status == 0 and crbias and clobber and os.path.isfile(masterbias)): 
	saltio.delete(masterbias,False,logfile)

# create master bias frame

    if (status == 0 and crbias): 
	createmaster(masterbias,mbiasdata,outfiles,instrume,amplifiers,plotover,verbose,logfile)

# end time

    if (status == 0):
	saltprint.time('\nSALTBIAS -- completed at',logfile,verbose)
    else:
	saltprint.time('\nSALTBIAS -- aborted at',logfile,verbose)

# ------------------------------------
# create the master bias frame

def createmaster(masterbias,mbiasdata,outfiles,instrume,amplifiers,plotover,verbose,logfile):

# open first FITS file in list of bias frames

    struct,status = saltio.openfits(outfiles[0],logfile)

# iterate over image HDUs, read images

    saltprint.log(logfile,' ',verbose)
    for amplifier in range(1,amplifiers+1):
	imagedata, status = saltio.readimage(struct,amplifier,logfile)

# iterate over images im the bias frame list

	tmpimage = []
	for biasframe in range(len(mbiasdata)):
	    if (biasframe%amplifiers + 1 == amplifier):
		tmpimage.append(mbiasdata[biasframe])

# average bias frames

	imagedata, status = saltstat.median_combine(tmpimage,logfile)
	message = 'Median averaging amplifier '+str(amplifier)+' for ' + masterbias.split('/')[-1]
	saltprint.log(logfile,message,verbose)

# plot master bias images

	if (plotover):
	    figure(2)
	    if (amplifier == amplifiers/2+1):
                title('                    PySALT -- SALTBIAS: '+masterbias.split('/')[-1])
	    if (instrume == 'RSS'):
		bx = 0.02+(amplifier-1)*0.163
		dx = 0.1
	    elif (instrume == 'SALTICAM'):
		bx = 0.02+(amplifier-1)*0.24
		dx = 0.17
	    by = 0.1
	    dy = 0.8
	    axes([bx,by,dx,dy])
	    setp(gca(),xticklabels=[],yticklabels = [])
	    jet()
	    ion()
	    imshow(imagedata)
	    colorbar()
	    ioff()

# append average image to master bias

	if (status == 0): struct,status = saltio.writeimage(struct,amplifier,imagedata,logfile)

# write master bias file to disk

    message = '\nSALTBIAS -- created master bias ' + masterbias.split('/')[-1] + ' at'
    if (status == 0): status = saltio.writefits(struct,masterbias,logfile)
    if (status == 0): status = saltio.closefits(struct,logfile)
    saltprint.time(message,logfile,verbose)

# ------------------------------------
# subtract bias

def bias(struct,infile,outfile,ifil,subbias,crbias,subover,plotover,median,
	 function,order,rej_lo,rej_hi,niter,trim,masterbias,bstruct,mbiasdata,
	 biasgn,biassp,biasbn,biasin,filetime,biastime,turbo,history,logfile,
	 verbose,status):

# plot colours

    linecol = {}
    linecol[1] = 'b' 
    linecol[2] = 'g' 
    linecol[3] = 'r' 
    linecol[4] = 'c' 
    linecol[5] = 'm' 
    linecol[6] = 'y' 
    trimyn = 'N'
    if (trim): trimyn = 'Y'
    medianyn = 'n'
    if (median): medianyn = 'y'

# identify instrument

    if (status == 0): instrume,keyprep,keygain,keybias,keyxtalk,keyslot,status = \
	saltkey.instrumid(struct,infile,logfile)

# has file been prepared?

    if (status == 0): status = saltkey.prepare(struct,infile,keyprep,logfile)

# how many extensions?

    if (status == 0): nextend,status = saltkey.get('NEXTEND',struct[0],infile,logfile)

# how many amplifiers?

    if (status == 0): 
	amplifiers,status = saltkey.get('NCCDS',struct[0],infile,logfile)
	amplifiers *= 2

# if master bias is to be subtracted - check data and bias are compatible

    if (status == 0 and subbias):
	gainset, status = saltkey.get('GAINSET',struct[0],infile,logfile)
	rospeed, status = saltkey.get('ROSPEED',struct[0],infile,logfile)
	ccdsum, status = saltkey.get('CCDSUM',struct[0],infile,logfile)
	if (gainset != biasgn):
	    message = 'ERROR -- SALTBIAS: gain in '+infile+'='+gainset
	    message += ', but gain in '+masterbias+'='+biasgn
	    status = saltprint.err(logfile,message)
	if (rospeed != biassp):
	    message = 'ERROR -- SALTBIAS: speed in '+infile+'='+rospeed
	    message += ', but speed in '+masterbias+'='+biassp
	    status = saltprint.err(logfile,message)
	if (ccdsum != biasbn):
	    message = 'ERROR -- SALTBIAS: binning in '+infile+'='+ccdsum
	    message += ', but binning in '+masterbias+'='+biasbn
	    status = saltprint.err(logfile,message)
	if (instrume != biasin):
	    message = 'ERROR -- SALTBIAS: '+infile+'contains '+instrume
	    message += ' data, but '+masterbias+' contains '+biasin+' data'
	    status = saltprint.err(logfile,message)
		
# if master bias is to be created - readout speed, gain, binning and name of 
# each exposure. check all exposures are called 'BIAS' and have consistent 
# keywords

    if (status == 0 and crbias):
	object, status = saltkey.get('OBJECT',struct[0],infile,logfile)
	gainset, status = saltkey.get('GAINSET',struct[0],infile,logfile)
	rospeed, status = saltkey.get('ROSPEED',struct[0],infile,logfile)
	ccdsum, status = saltkey.get('CCDSUM',struct[0],infile,logfile)
	object = object.strip()
	if (ifil == 0):
	    biasgn = gainset
	    biassp = rospeed
	    biasbn = ccdsum
	if (object != 'BIAS'):
	    message = 'WARNING -- SALTBIAS: if crbias = \'yes\', all '
	    message += 'exposures must be bias frames. OBJECT keyword '
	    message += '\''+object+'\' does not suggest '+infile
	    message += ' is a bias frame\n'
	    saltprint.log(logfile,message,verbose)
	if (gainset != biasgn):
	    message = 'ERROR -- SALTBIAS: bias frame gains are not all '
	    message += 'identical. GAINSET=\''+gainset+'\''
	    status = saltprint.err(logfile,message)
	if (rospeed != biassp):
	    message = 'ERROR -- SALTBIAS: bias frame readout speeds '
	    message += 'are not all identical. ROSPEED=\''+rospeed+'\''
	    status = saltprint.err(logfile,message)
	if (ccdsum != biasbn):
	    message = 'ERROR -- SALTBIAS: bias frame binnings are not '
	    message += 'all identical. CCDSUM=\''+ccdsum+'\''
	    status = saltprint.err(logfile,message)

# date, time and dwell of each file

    if (status == 0):
	dateobs,status = saltkey.get('DATE-OBS',struct[0],infile,logfile)
	utcobs, status = saltkey.get('UTC-OBS',struct[0],infile,logfile)
	exptime, status = saltkey.get('EXPTIME',struct[0],infile,logfile)
	if (len(dateobs) == 0 or len(utcobs) == 0 or len(str(exptime)) == 0):
	    status = 1
    if (status == 0):
	year = int(dateobs.split('-')[0])
	month = int(dateobs.split('-')[1])
	day = int(dateobs.split('-')[2])
	hour = int(utcobs.split(':')[0])
	min = int(utcobs.split(':')[1])
	sec = int(float(utcobs.split(':')[2]))
	timeobs = float(hour)+float(min)/60+(float(sec)+exptime/2)/3600
	if (timeobs < 9.0): timeobs += 24.0

# housekeeping keywords

    if (status == 0): 
	status = saltkey.housekeeping(struct[0],keybias, 'Images have been de-biased', history,outfile,logfile)

# log overscan table

    if (status == 0 and subover):
	message = '%28s %7s %5s %4s %6s' % \
	    ('HDU','Overscan','Order','RMS','Niter')
	saltprint.log(logfile,'\n     --------------------------------------------------',verbose)
	saltprint.log(logfile,message,verbose)
	saltprint.log(logfile,'     --------------------------------------------------',verbose)

# if using colbias, create output file

    if (status == 0 and function != 'polynomial'):
	status = saltio.writefits(struct[0],outfile,logfile)

# loop over image extensions, determine overscan and data locations

    if (status == 0):
	for i in range(nextend):
	    hdu = i + 1
	    amplifier = hdu%amplifiers
	    if (amplifier == 0): amplifier = amplifiers
	    if (status == 0): biassec, status = saltkey.get('BIASSEC',struct[hdu],infile,logfile)
	    if (status == 0):
		ranges = biassec.lstrip('[').rstrip(']').split(',')
		x1 = int(ranges[0].split(':')[0]) - 1
		x2 = int(ranges[0].split(':')[1]) - 1
		y1 = int(ranges[1].split(':')[0]) - 1
		y2 = int(ranges[1].split(':')[1]) - 1
	    if (status == 0): datasec,status = saltkey.get('DATASEC',struct[hdu],infile,logfile)
	    if (status == 0):
		ranges = datasec.lstrip('[').rstrip(']').split(',')
		dx1 = int(ranges[0].split(':')[0]) - 1
		dx2 = int(ranges[0].split(':')[1])
		dy1 = int(ranges[1].split(':')[0]) - 1
		dy2 = int(ranges[1].split(':')[1])

# read image from HDU structure
		    
	    if (status == 0):
                if (plotover): figure(1)
		value,status = saltkey.get('NAXIS1',struct[hdu],infile,logfile)
		naxis1 = int(value)
		value,status = saltkey.get('NAXIS2',struct[hdu],infile,logfile)
		naxis2 = int(value)
		imagedata,status = saltio.readimage(struct,hdu,logfile)

# collapse and plot overscan in y-direction

	    if (status == 0 and (plotover or turbo)):
		overx = []
		overx_val = []
		for x in range(x1,x2+1):
		    list = imagedata[y1:y2,x] * 1.0
		    overx.append(x-x1+1)
		    if (median):
			overx_val.append(saltstat.median(list,logfile))
		    else:
			overx_val.append(saltstat.mean(list))
		if (status == 0 and plotover):
		    axes([.1,.1,.2,.5])
		    if (amplifier == 1): cla()
		    xlabel('CCD Column')
		    ylabel('Pixel Counts (e-)')
		    ion()
		    plot(overx,overx_val,linestyle='',markersize=8,marker='.')
		    ioff()

# collapse overscan in x-direction and fit

	    overy = []
	    overy_val = []
	    if (status==0):
	    	for y in range(y1,y2+1):
		  list = imagedata[y,x1:x2] * 1.0
		  overy.append(y)
		  if (median):
		    overy_val.append(saltstat.median(list,logfile))
		  else:
		    overy_val.append(saltstat.mean(list))
	    	overy_val[len(overy_val)-1] = overy_val[len(overy_val)-2]
	    	if (median):
		  coeffs = polyfit(overy,overy_val,order)
		  iterations = 0
	    	else:
		  coeffs,iterations = saltfit.poly(overy,overy_val,order,
						 rej_lo,rej_hi,niter)
	    	if (function == 'polynomial'): ofit = polyval(coeffs,overy)

# plot overscan data and fit
	    
	    if (status == 0 and plotover):
		axes([.37,.1,.58,.5])
		if (amplifier == 1): cla()
		xlabel('CCD Row')
		ion()
		plot(overy,overy_val,linestyle='',markersize=1,marker='.',
		     markerfacecolor=linecol[amplifier],markeredgecolor=linecol[amplifier])
		if (function == 'polynomial'): 
		    plot(overy,ofit,linewidth=1,linestyle='-',marker='',color=linecol[amplifier])
		if (instrume == 'RSS'):
		    legend(('','Amp1','','Amp2','','Amp3','','Amp4','','Amp5','','Amp6'),
			   prop = matplotlib.font_manager.FontProperties(size=6),loc='lower right')
		else:
		    legend(('','Amp1','','Amp2','','Amp3','','Amp4'),
			   prop = matplotlib.font_manager.FontProperties(size=6),loc='lower right')
		ioff()

# plot bias level history 

	    if (status == 0 and plotover):
		try:
		    dateobs = struct[hdu].header['DATE-OBS']
		    utcobs = struct[hdu].header['UTC-OBS']
		    year = int(dateobs.split('-')[0])
		    month = int(dateobs.split('-')[1])
		    day = int(dateobs.split('-')[2])
		    hour = int(utcobs.split(':')[0])
		    min = int(utcobs.split(':')[1])
		    sec = float(utcobs.split(':')[2])
		    timeobs = float(hour)+float(min)/60+(float(sec)+exptime/2)/3600
		    if (timeobs < 9.0): timeobs += 24.0
		except:
		    timeobs = timeobs
	    if (status == 0 and plotover and median):
		overlevel = saltstat.median(overx_val,logfile)
	    elif (status == 0 and plotover and not median):
		overlevel = saltstat.mean(overx_val)
	    elif (status == 0 and not plotover and median):
		overlevel = saltstat.median(overy_val,logfile)
	    else:
	       if (status == 0): 
		overlevel = saltstat.mean(overy_val)
	    if (status == 0 and plotover):
		filetime[amplifier].append(timeobs)
		biastime[amplifier].append(overlevel)
		if (len(filetime[amplifier]) > 1):
		    axes([.1,.68,.85,.25])
		    title('PySALT -- SALTBIAS: '+infile)
		    xlabel('UT (hrs)')
		    ylabel('Pixel Counts (e-)')
		    ion()
		    plot(filetime[amplifier],biastime[amplifier],
			 linestyle='-',markersize=8,marker='.',
			 color=linecol[amplifier],
			 markeredgecolor=linecol[amplifier],
			 markerfacecolor=linecol[amplifier])
		    ioff()

# statistics of overscan fit

	    if (status == 0 and function == 'polynomial'):
		sigma = 0
		overmean = 0
		for iy in range(len(overy)):
		    overmean += overy_val[iy] - ofit[iy]
		    sigma += (overy_val[iy] - ofit[iy])**2
		overmean /= len(overy)
		sigma = sqrt(sigma / (len(overy) - 1))
		if (median): overmedian = saltstat.median(overy_val-ofit,logfile)

# use noao.imred.bias.colbias for different function fits

	    if (status == 0 and function != 'polynomial'):
		tmplog, status = saltio.tmpfile('.',False,logfile)
		if (status == 0): 
		    try:
			iraf.noao.imred.bias.colbias(input=infile + '[' + str(hdu) + ']',
						     output=outfile + '[' + str(hdu) + ']',
						     bias=biassec,
						     trim=datasec,
						     median=medianyn,
						     interactive='n',
						     function=function,
						     order=order,
						     low_reject=rej_lo,
						     high_reject=rej_hi,
						     niterate=niter,
						     logfiles=tmplog,
						     graphics='stdgraph')
		    except:
			message = 'ERROR -- SALTBIAS: cannot call iraf.imred.bias.colbias'
			status = saltprint.err(logfile,message)

# extract fit rms from colbias temporary log

	    if (status == 0 and function != 'polynomial'):
		tmplines, status = saltio.openascii(tmplog,'r',logfile)
		for tmpline in tmplines:
		    tmpline = tmpline.split('=')
		    if (tmpline[0] == '# RMS '):
			sigma = float(tmpline[1].strip())
		status = saltio.delete(tmplog,False,logfile)

# log overscan data

	    if (status == 0 and subover and median and not turbo and function == 'polynomial'):
		status = saltkey.new('OVERSCAN','%f' % (overmedian),'Overscan median value',
				     struct[hdu],outfile,logfile)
	    elif (status == 0 and subover and not median and not turbo and function == 'polynomial'):
		status = saltkey.new('OVERSCAN','%f' % (overmean),'Overscan mean value',
				     struct[hdu],outfile,logfile)
	    if (status == 0 and subover and not turbo and function == 'polynomial'):
		status = saltkey.new('OVERRMS','%f' % (sigma),
				     'Overscan RMS value',struct[hdu],outfile,logfile)
	    elif (status == 0 and subover and not turbo and function != 'polynomial'):
		status = saltkey.mkheader(outfile+'['+str(hdu)+']','OVERRMS','%f' % (sigma),
					  'Overscan RMS value',False,logfile)
	    if (status == 0 and subover):
		tabfile = infile.split('/')
		message = '%25s[%1d] %8.2f %3d %7.2f %3d' % \
		    (tabfile[len(tabfile)-1], hdu, overlevel, order, sigma, iterations)
		saltprint.log(logfile,message,verbose)

# subtract overscan fit from image columns

	    if (status == 0):
		if (turbo):
		    imagedata -= overlevel
		else:
		    for icol in range(0,naxis1):
			if (function == 'polynomial'): imagedata[y1:y2+1,icol] -= ofit

# trim image of overscan and underscan regions

	    if (status == 0 and trim and function == 'polynomial'):
		imagedata = imagedata[dy1:dy2,dx1:dx2]
		datasec = '[1:'+str(dx2-dx1)+',1:'+str(dy2-dy1)+']'
		status = saltkey.put('DATASEC',datasec,struct[hdu],outfile,logfile)
#		status = saltkey.put('TRIMSEC',datasec,struct[hdu],outfile,logfile)
		status = saltkey.rem('BIASSEC',struct[hdu],outfile,logfile)

# if master bias is to be created keep image in memory for later

	    if (status == 0 and crbias):
		mbiasdata.append(imagedata)

# subtract master bias if appropriate

	    if (status == 0 and subbias):
		biasimage,status = saltio.readimage(bstruct,amplifier,logfile)
		bdatasec,status = saltkey.get('DATASEC',bstruct[amplifier],
					      masterbias,logfile)
		if (bdatasec == datasec):
		    imagedata -= biasimage
		else:
		    message = 'ERROR -- SALTBIAS: DATASEC keyword '
		    message += '(image size) is incompatible between '
		    message += infile+' and '+masterbias
		    status = saltprint.err(logfile,message)

# update image in HDU structure

	    if (status == 0 and function == 'polynomial'): 
		struct,status = saltio.writeimage(struct,hdu,imagedata,logfile)

    return struct, mbiasdata, biasbn, biassp, biasbn, biasgn, filetime, biastime, \
	amplifiers, instrume, status

# -----------------------------------------------------------
# main code 

parfile = iraf.osfn("saltred$saltbias.par") 
t = iraf.IrafTaskFactory(taskname="saltbias",value=parfile,function=saltbias, pkgname='saltred')
